//Passmark USB3 console program
//USB3_Console.h
//Copyright PassMark Software 2018
//http://www.passmark.com

#ifndef _USB3_CONSOLE_H
#define _USB3_CONSOLE_H


#include <windows.h>
#include <CyAPI.h>

// Application
#define PROGRAM_NAME	"USB3Console"	// Name of the application
#define PROGRAM_VERSION	"V1.0"			// Version of application
#define PROGRAM_BUILD	"1004"			// Build number within this version

#define LOOPBACK_USB3_VENDOR_ID     0x0403	//Vendor ID for PassMark USB3 PLugs
#define LOOPBACK_USB3_PRODUCT_ID    0xff0b	//Product ID for PassMark USB3 PLugs

#define MAX_USB_DEV_NUMBER			32		// Number of USB device driver instances possible

#define	ONE_MEGABYTE				1048576

#define	BENCHMARK_R_W_BUFFER_SIZE	49152
#define	BENCHMARK_R_W_BUFFER_COUNT	2
#define	BENCHMARK_RW_BUFFER_SIZE	24576
#define	BENCHMARK_RW_BUFFER_COUNT	2
#define	LOOPBACK_BUFFER_SIZE		1024
#define	LOOPBACK_BUFFER_COUNT		64
#define BENCHMARK_TRANSFER_SIZE		2097152

#define LPM_ENTRY_DISABLE			0
#define LPM_ENTRY_ENABLE			(1 << 8)

#define MAX_TRANSFER_SIZE_SS		2097152		// 2MB Based on trial and error on different machines with different host controllers (4M will failed on an old machine with ASMEDIA host controller)
#define MAX_TRANSFER_SIZE_HS		2097152		// To be compatible with EHCI (Max 3MB)
#define MAX_TRANSFER_SIZE_FS		262144		// To be compatible with OHCI
#define MAX_PACKET_PER_XFER_SS		40			// According to Cypress documents maximum transfer size is 4MB, 4194304 / (16 bursts) / (3 trans per interval) / (1K buffer size) = 85, it should be multipled of 8, so 80
#define MAX_PACKET_PER_XFER_HS		128			// According to Cypress documents maximum is 128, microsoft says 1024 
#define MAX_PACKET_PER_XFER_FS		64			// Limited by UHCI, OHCI controllers
#define MAXQUEUESZ					64

#define DISPLAY_DISABLE	 0
#define DISPLAY_ENABLE	 (1 << 8)

#define	PHY_ERROR_DECODE_EV		(1 << 0)
#define	PHY_ERROR_EB_OVR_EV		(1 << 1)
#define	PHY_ERROR_EB_UND_EV		(1 << 2)
#define	PHY_ERROR_DISPARITY_EV	(1 << 3)
#define	RX_ERROR_CRC5_EV		(1 << 4)
#define	RX_ERROR_CRC16_EV		(1 << 5)
#define	RX_ERROR_CRC32_EV		(1 << 6)
#define	TRAINING_ERROR_EV		(1 << 7)
#define	PHY_LOCK_EV				(1 << 8)
#define PHY_ERR_EN_ALL			0x1ff

#define	HP_TIMEOUT_EV			(1 << 0)
#define	RX_SEQ_NUM_ERR_EV		(1 << 1)
#define	RX_HP_FAIL_EV			(1 << 2)
#define	MISSING_LGOOD_EV		(1 << 3)
#define	MISSING_LCRD_EV			(1 << 4)
#define	CREDIT_HP_TIMEOUT_EV	(1 << 5)
#define	PM_LC_TIMEOUT_EV		(1 << 6)
#define	TX_SEQ_NUM_ERR_EV		(1 << 7)
#define	HDR_ADV_TIMEOUT_EV		(1 << 8)
#define	HDR_ADV_HP_EV			(1 << 9)
#define	HDR_ADV_LCRD_EV			(1 << 10)
#define	HDR_ADV_LGO_EV			(1 << 11)
#define	CREDIT_ADV_TIMEOUT_EV	(1 << 12)
#define	CREDIT_ADV_HP_EV		(1 << 13)
#define	CREDIT_ADV_LGO_EV		(1 << 14)
#define LNK_ERR_EN_ALL			0x7fff

#define ERR_EN_ALL				((LNK_ERR_EN_ALL << 16L) | PHY_ERR_EN_ALL);


typedef enum _USB30_VENDOR_REQ_TYPE
{
	VENDOR_REQ_READ = 0,
	VENDOR_REQ_WRITE,
} USB30_VENDOR;

enum
{
	CHANGELEDS = 1,
	SET_CONFIG,
	GET_CONFIG,
	SET_DISP_MODE,
	CONF_ERROR_COUNTERS,
	GET_ERROR_COUNTS,
	GET_VOLTAGE,
	DELETE_FIRMWARE = 0x08,
	GET_MAX_SPEED,
	RESET_ERROR_COUNTERS,
	CONFIG_LPM_ENTRY,
	GET_DEVICE_INFO = 0x50,
	GET_PRODUCTION_CODE = 0x51
}USB30_VENDOR_CMD;

typedef enum _DATA_PATTERN
{
	INCREMENTINGBYTE = 0,
	RANDOMBYTE,
	CONSTANTBYTE
} DATA_PATTERN;


typedef enum _TEST_STATUS
{
	TEST_STATUS_SUCCEED = 0,
	TEST_STATUS_FAILED_TO_FIND_ENDPOINTS,
	TEST_STATUS_FAILED_TO_ALLOCATE_MEM,
	TEST_STATUS_ERROR_LIMIT_REACHED,
} TEST_STATUS;

typedef enum _CONSOLE_STATUS
{
	STATUS_SUCCEED = 0,
	STATUS_INVALID_COMMAND_LINE,
	STATUS_NO_DEVICE_DETECTED,
	STATUS_FAILED_TO_CONNECT,
	STATUS_FAILED_TO_CONFIGURE,
	STATUS_FAILED_TO_SELECT_SPEED, 
	STATUS_FAILED_TO_FIND_ENDPOINTS,
	STATUS_FAILED_TO_ALLOCATE_MEM,
	STATUS_ERROR_LIMIT_REACHED,
} CONSOLE_STATUS;

typedef enum _USB30_SPEED
{
	USB30ST_UNKNOWN = 0,
	USB30ST_FS,
	USB30ST_HS,
	USB30ST_SS,

} USB30_SPEED;

typedef enum _TEST_MODE
{
	TEST_LOOPBACK = 0,
	TEST_BENCHMARK,
} TEST_MODE;

typedef enum _BENCHMARK_MODE
{
	BENCHMARK_READ = 0,
	BENCHMARK_WRITE,
} BENCHMARK_MODE;

typedef enum _BENCHMARK_TYPE
{
	BENCHMARK_RTW = 0,			//Read then write 
	BENCHMARK_RO,				//Read only
	BENCHMARK_WO,				//Write only
	BENCHMARK_RW,				//Read and write (same time)
} BENCHMARK_TYPE;

typedef enum
{
	USB_EP_CONTROL = 0,          /* Control Endpoint Type */
	USB_EP_ISOCHRONOUS = 1,      /* Isochronous Endpoint Type */
	USB_EP_BULK = 2,             /* Bulk Endpoint Type */
	USB_EP_INTERRUPT = 3         /* Interrupt Endpoint Type */
} USB_EP_TYPE;

typedef enum
{
	DMA_LOOPBACK_AUTO = 0,		/* Control Endpoint Type */
	DMA_BENCHMARK_READ = 1,		/* Isochronous Endpoint Type */
	USB_BENCHMARK_WRITE = 2,	/* Bulk Endpoint Type */
	USB_BENCHMARK_RW = 3		/* Interrupt Endpoint Type */
} USB_TEST_TYPE;

typedef struct
{
	UCHAR	dma_mode;
	UCHAR	ep_type;
	UCHAR	ep_in;
	UCHAR	ep_out;
	UCHAR	ss_burst_len;
	UCHAR	polling_interval;
	UCHAR	hs_bulk_nak_interval;
	UCHAR	iso_transactions_per_bus_interval;
	UINT16	iso_bytes_per_bus_interval;
	// settings added from firmware version 1.8
	UCHAR	speed;
	UCHAR	buffer_count;
	UINT16	buffer_size;
} config_info_def;

typedef struct
{
	wchar_t	serial[USB_STRING_MAXLEN];	//Serial number of attached plug
	int		deviceNum;					//Device number

} PassMarkUSB3Plug;

typedef struct
{
	LONG				TransferSize;
	LONG				TestDuration;
	WORD				ErrorLimit;
	bool				VerifyData;
	bool				CheckBusErrors;
	DATA_PATTERN		DataPattern;
	UCHAR				ConstantByte;
	USB30_SPEED			Speed;
	BENCHMARK_TYPE		BenchmarkType;
} TestParams;

typedef struct
{
	time_t	TestStartTime;		//Test start time
	time_t	TestDuration;		//Test start time
	UINT64	TestCycle;			//Test cycle count 
	UINT64	TotReadBytes;		//Total bytes read during test
	UINT64	TotWriteBytes;		//Total bytes written during test
	double	AvgRate;			//Average read & write rate
	double	AvgReadRate;		//Average read rate
	double	AvgWriteRate;		//Average write rate
	UINT64	Errors;				//Other error count
	UINT64	ReadErrors;			//Read error count
	UINT64	WriteErrors;		//Write error count
	UINT64	PhysicalErrors;		//Physical error count
} TestResults;


#if defined (MAIN_MODULE) 
PassMarkUSB3Plug	PassMarkPlugs[MAX_USB_DEV_NUMBER];		//Serial numbers andtheior matching device number of the currently connected plugs
CCyUSBDevice		*CurUSBDevice = NULL;					//Currently open USB3 plug
config_info_def		usb3Config;
CCyUSBEndPoint*		InEndpt = NULL;
CCyUSBEndPoint*		OutEndpt = NULL;
#else
extern PassMarkUSB3Plug		PassMarkPlugs[MAX_USB_DEV_NUMBER];	//Serial numbers andtheior matching device number of the currently connected plugs
extern CCyUSBDevice			*CurUSBDevice;						//Currently open USB3 plug
extern config_info_def		usb3Config;
extern	CCyUSBEndPoint*		InEndpt;
extern	CCyUSBEndPoint*		OutEndpt;
#endif

#endif
